function DebugLogger(){
	this.name;
	this.paused = false;
	this.bufferSize = 250;
	this.logs = new Array();
	this.notifier = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
	
	/**
	* Initialize the logger
	* @param string name Name of project to register
	*/
	this.init = function(name){
		try{
			this.name = name;
		}catch(e){
			throw "nsIDebugLogger(init)\n"+ e;
		}
	}
	
	/**
	* Add log
	* @param int level Level of debug (1=most granular, 5=most broad)
	* @param string message Message to log
	*/
	this.log = function(level, message){
		try{
			if(this.paused == true){
				return;
			}
			
			log = Components.classes["@mozmonkey.com/debuglogger/log;1"].createInstance(Components.interfaces.nsIDebugLoggerLog);
			log.init(level, message, this);
			this.logs.push(log);
			
			// Adjust history buffer size
			var oPrefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
			if(oPrefs.prefHasUserValue("debuglogger.buffer") && oPrefs.getIntPref("debuglogger.buffer") > 50){
				this.bufferSize = oPrefs.getIntPref("debuglogger.buffer");
			}
			while(this.logs.length > this.bufferSize){
				this.logs.shift();
			}
			
			this.notifier.notifyObservers(null, "debug-logger-add", this.name);
		}catch(e){
			throw "nsIDebugLogger(log)\n"+ e;
		}
	}
	
	/**
	* Get all logs
	* @param array count An empty array
	* @return array Array of nsIDebugLoggerLog objects
	*/
	this.getLogs = function(count){
		try{
			count.value = this.logs.length;
			return this.logs;
		}catch(e){
			throw "nsIDebugLogger(getLogs)\n"+ e;
		}
	}
	
	/**
	* Return last log entry\
	* @return nsIDebugLoggerLog log
	*/
	this.getLastLog = function(){
		if(this.logs.length > 0){
			return this.logs[this.logs.length - 1];
		}
		return null;
	}
	
	/**
	* Clear all logs
	*/
	this.clearLogs = function(){
		try{
			this.logs = new Array();
			this.notifier.notifyObservers(null, "debug-logger-clear", this.name);
		}catch(e){
			throw "nsIDebugLogger(clearLogs)\n"+ e;
		}
	}
	
	/**
	* Pause this logger
	*/
	this.pause = function(){
		this.paused = true;
	}
	
	/**
	* Resume this logger
	*/
	this.resume = function(){
		this.paused = false;
	}
}

DebugLogger.prototype = {

	QueryInterface: function (iid) {
		if(!iid.equals(Components.interfaces.nsIDebugLogger) && !iid.equals(Components.interfaces.nsISupports))
			throw Components.results.NS_ERROR_NO_INTERFACE;
		
		return this;
	}
};
var Module = {
	myCID : Components.ID("{4C0BFB4A-598C-43a8-847E-7A0AF8669FA1}"),
    myProgID : "@mozmonkey.com/debuglogger/logger;1",
    firstTime : true,
	registerSelf : function (compMgr, fileSpec, location, type) {
		if(this.firstTime) {
			this.firstTime = false;
			throw Components.results.NS_ERROR_FACTORY_REGISTER_AGAIN;
		}
		compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
		compMgr.registerFactoryLocation(this.myCID, "Debug Logger", this.myProgID, fileSpec, location, type);
	},
	getClassObject : function (compMgr, cid, iid) {
		if (!cid.equals(this.myCID)) throw Components.results.NS_ERROR_NO_INTERFACE;
		if (!iid.equals(Components.interfaces.nsIFactory)) throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
		return this.myFactory;
	},
    myFactory : {
		createInstance : function (outer, iid) {
			if (outer != null) throw Components.results.NS_ERROR_NO_AGGREGATION;
			return (new DebugLogger()).QueryInterface(iid);
		}
	},
    canUnload : function(compMgr) { return true; }
};
function NSGetModule(compMgr, fileSpec) { return Module; }