function DebugLoggerManager(){ }

DebugLoggerManager.prototype = {
	paused : false,
	loggers : new Array(),
	
	/**
	* Add a new logger by name
	* @return nsIDebugLogger
	*/
	registerLogger : function(name){
		try{
			if(name.toLowerCase() == "__all__"){
				throw "'__all__' is a reserved name and cannot be registered";
			}
			else if(name.toLowerCase() == "all"){
				throw "'all' is a reserved name and cannot be registered";
			}
			else if( !(new RegExp("\\S", "g").test(name)) ){
				throw "The logger name cannot be empty";
			}
		
			if(typeof(this.loggers[name]) == 'undefined'){
				this.loggers[name] = Components.classes["@mozmonkey.com/debuglogger/logger;1"].createInstance(Components.interfaces.nsIDebugLogger);
				this.loggers[name].init(name, this);
				
				if(this.paused){
					this.loggers[name].pause();
				}
				
				var notifier = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
					notifier.notifyObservers(null, "debug-logger-register", name);
			}
			return this.loggers[name];
		}catch(e){
			throw "nsIDebugLoggerManager(registerLogger)\n"+ e;
		}
	},
	
	/**
	* Get a logger by name
	* @return nsIDebugLogger
	*/
	getLogger : function(name){
		try{
			if(typeof(this.loggers[name]) != 'undefined'){
				return this.loggers[name];
			}
		}catch(e){
			throw "nsIDebugLoggerManager(getLogger)\n"+ e;
		}
		return null;
	},
	
	/**
	* Is a logger by this name registered?
	* @param string name
	* @return boolean
	*/
	hasLogger : function(name){
		try{
			if(typeof(this.loggers[name]) != 'undefined'){
				return true;
			}
		}catch(e){
			throw "nsIDebugLoggerManager(hasLogger)\n"+ e;
		}
		return false;
	},
	
	/**
	* Get list of all loggers
	* @param array count An empty array
	* @return array Array of nsIDebugLogger objects
	*/
	getLoggers : function(count){
		try{
			var out = new Array();
			for(name in this.loggers){
				out.push(this.loggers[name]);
			}
		
			count.value = out.length;
			return out;
		}catch(e){
			throw "nsIDebugLoggerManager(getLoggers)\n"+ e;
		}
	},
	
	/**
	* Pause all loggers
	*/
	pause : function(){
		try{		
			for(name in this.loggers){
				this.loggers[name].pause();
			}
			this.paused = true;
		}catch(e){
			throw "nsIDebugLoggerManager(pause)\n"+ e;
		}
	},
	
	/**
	* Resume all loggers
	*/
	resume : function(){
		try{
			for(name in this.loggers){
				this.loggers[name].resume();
			}
			this.paused = false;
		}catch(e){
			throw "nsIDebugLoggerManager(resume)\n"+ e;
		}
	},
	
	QueryInterface : function (iid) {
		if(!iid.equals(Components.interfaces.nsIDebugLoggerManager) && !iid.equals(Components.interfaces.nsISupports))
			throw Components.results.NS_ERROR_NO_INTERFACE;
		
		return this;
	}
};
var Module = {
	myCID : Components.ID("{30267C28-8DD4-46c4-B47B-683E8C757A08}"),
    myProgID : "@mozmonkey.com/debuglogger/manager;1",
    firstTime : true,
	registerSelf : function (compMgr, fileSpec, location, type) {
		if(this.firstTime) {
			this.firstTime = false;
			throw Components.results.NS_ERROR_FACTORY_REGISTER_AGAIN;
		}
		compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
		compMgr.registerFactoryLocation(this.myCID, "Debug Log Manager", this.myProgID, fileSpec, location, type);
	},
	getClassObject : function (compMgr, cid, iid) {
		if (!cid.equals(this.myCID)) throw Components.results.NS_ERROR_NO_INTERFACE;
		if (!iid.equals(Components.interfaces.nsIFactory)) throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
		return this.myFactory;
	},
    myFactory : {
		createInstance : function (outer, iid) {
			if (outer != null) throw Components.results.NS_ERROR_NO_AGGREGATION;
			return (new DebugLoggerManager()).QueryInterface(iid);
		}
	},
    canUnload : function(compMgr) { return true; }
};
function NSGetModule(compMgr, fileSpec) { return Module; }